<template>
  <div class="mod-config">
    <basic-container>
      <ttable
        :table="table"
        @handleSelectionChange="handleSelectionChange"
        :uploadInfo="uploadInfo"
        :prelang="prelang"
        :options="options"
        :ajaxFun="ajaxFun"
        ref="completeProductStructureTable"
      >
        <template #toolbar></template>
      </ttable>
    </basic-container>
  </div>
</template>

<script>
import {
  fetchList,
  delObj,
  addByStructure
} from '@/api/technology/completeproductstructure'
import ttable from '@/views/common/ztt-table.vue'
import { mapGetters } from 'vuex'
import { changeState } from '../../../api/technology/completeproductstructure'
import { remote } from '@/api/admin/dict'
export default {
  data() {
    return {
      ajaxFun: fetchList,
      multipleSelection: [],
      uploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: true,
        url: 'mes/bom/uploadExcel'
      },
      prelang: 'completeProductStructure',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮
        isSearch: false, // 高级查询按钮
        defaultOrderBy: { column: 'id', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          // 工序编号
          {
            minWidth: '140',
            prop: 'number',
            label: 'BOM编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            render: { fun: this.addOrUpdateHandle }
          },
          // 零件名称
          {
            minWidth: '120',
            prop: 'partName',
            label: '零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 零件号
          {
            minWidth: '120',
            prop: 'partNo',
            label: '零件编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },

          // 状态
          {
            minWidth: '120',
            prop: 'state',
            label: '状态',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            optList: () => {
              return this.getStateOptionList()
            },
            formatter: (row, column, cellValue) => {
              var formatVal
              if (cellValue == '01draft') {
                formatVal = '草稿'
              } else if (cellValue == '02accepted') {
                formatVal = '已接受'
              } else {
                formatVal = '已取消'
              }
              return formatVal
            }
          },

          // 类型
          {
            minWidth: '120',
            prop: 'bomTypeDb',
            label: '类型',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatBomTypeDbType,
            optList: () => {
              return this.bomTypeDbOptions
            }
          },
          // 版本号
          {
            minWidth: '120',
            prop: 'version',
            label: '版本号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },

          // 替代号
          {
            minWidth: '120',
            prop: 'alternativeNo',
            label: '替代号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },

          // 替代描述
          {
            minWidth: '120',
            prop: 'alternativeDesc',
            label: '替代描述',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },

          // 绝缘颜色
          {
            minWidth: '120',
            prop: 'insulationColor',
            label: '绝缘颜色',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 护套颜色
          {
            minWidth: '120',
            prop: 'sheathColor',
            label: '护套颜色',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 特性1
          {
            minWidth: '120',
            prop: 'characteristicOne',
            label: '特性1',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 创建人
          {
            minWidth: '120',
            prop: 'createUser',
            label: '创建人',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 创建时间
          {
            minWidth: '200',
            prop: 'createTime',
            label: '创建时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange'
          }
        ],
        toolbar: [
          {
            text: '新增',
            type: 'primary',
            fun: this.addOrUpdateHandle,
            disabled: false,
            permitArr: []
          },
          {
            text: '产品结构生成BOM',
            fun: this.addByStructureHandle,
            disabled: false,
            permitArr: []
          },
          {
            text: '批准',
            fun: () => {
              this.approveHandle('ACCEPT')
            },
            disabled: false,
            permitArr: ['01draft']
          },
          {
            text: '撤回',
            fun: () => {
              this.approveHandle('REVOKE')
            },
            disabled: false,
            permitArr: ['02accepted']
          },
          {
            text: '拒绝',
            fun: () => {
              this.approveHandle('CANCEL')
            },
            disabled: false,
            permitArr: ['02accepted']
          },
          {
            text: '删除',
            fun: () => {
              this.deleteAll()
            },
            disabled: false,
            permitArr: []
          }
        ],
        operator: [
          {
            text: '删除',
            type: 'text',
            size: 'small',
            fun: this.deleteHandle
          }
        ],
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 100,
          minWidth: 100
        }
      },
      stateOptionList: [
        {
          value: '01draft',
          label: '草稿'
        },
        {
          value: '02accepted',
          label: '已接受'
        },
        {
          value: '03cancelled',
          label: '已取消'
        }
      ],
      bomTypeDbOptions: []
    }
  },
  components: {
    ttable
  },
  computed: {
    ...mapGetters(['permissions'])
  },
  activated() {
    this.getData()
  },
  created() {
    this.getBomTypeDbOptions()
  },
  methods: {
    getBomTypeDbOptions() {
      remote('bom_type_db').then((response) => {
        if (response.data.code === 0) {
          this.bomTypeDbOptions = response.data.data
        } else {
          this.bomTypeDbOptions = []
        }
      })
    },
    // 工艺类型中文格式化
    formatBomTypeDbType(row, column, cellValue) {
      this.bomTypeDbOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    // 获取数据列表
    getData() {
      if (this.$refs.completeProductStructureTable !== undefined) {
        this.$refs.completeProductStructureTable.getDataList()
      }
    },
    // 新增 / 修改
    addOrUpdateHandle(row) {
      this.$router.push({
        name: 'completeProductStructure',
        params: { id: row == null ? null : row.id }
      })
    },
    // 按产品结构生成BOM
    addByStructureHandle() {
      addByStructure()
      this.$message({
        message: '已在后台生成BOM，请执行结束后刷新页面',
        type: 'success'
      })
    },
    // 删除
    deleteHandle(row) {
      this.$confirm('是否确认删除编号为' + row.number, '提示', {
        confirmButtonText: '确定',
        cancelButtonText: '取消',
        type: 'warning',
        closeOnClickModal: false
      })
        .then(function() {
          const ids = []
          ids.push(row.id)
          return delObj(ids)
        })
        .then((response) => {
          var data = response.data
          if (data.code === 0) {
            this.$message.success('删除成功')
          } else {
            this.$message.error('删除失败')
          }
          this.getData()
        })
    },
    deleteAll() {
      const that = this
      if (this.multipleSelection.length > 0) {
        this.$confirm('是否确认删除', '提示', {
          confirmButtonText: '确定',
          cancelButtonText: '取消',
          type: 'warning',
          closeOnClickModal: false
        })
          .then(function() {
            const ids = that.multipleSelection.map((item) => {
              return item.id
            })
            return delObj(ids)
          })
          .then((response) => {
            var data = response.data
            if (data.code === 0) {
              this.$message.success('删除成功')
            } else {
              this.$message.error('删除失败')
            }
            this.getData()
          })
      } else {
        this.$message.error('请选择需要删除的对象')
      }
    },
    // 审批
    approveHandle(event) {
      var eventName
      if (event == 'ACCEPT') {
        eventName = '批准'
      } else if (event == 'REVOKE') {
        eventName = '撤回'
      } else {
        eventName = '拒绝'
      }
      if (this.multipleSelection.length > 0) {
        changeState(
          this.multipleSelection.map((item) => {
            return item.id
          }),
          event
        ).then((response) => {
          var data = response.data
          if (data.code == 0) {
            this.$message.success(eventName + '成功')
            this.getData()
          } else {
            this.$message.error(eventName + '失败')
          }
        })
      } else {
        this.$message.error('请选择' + eventName + '的对象')
      }
    },
    handleSelectionChange(val) {
      // 根据状态，禁用表头按钮
      // 筛选出选中记录的状态
      var stateArr = val.map(function(value, index) {
        return value.state
      })
      // 选中状态数组元素去重
      var uniqueStateArr = []
      for (var i = 0; i < stateArr.length; i++) {
        if (uniqueStateArr.indexOf(stateArr[i]) == -1) {
          uniqueStateArr.push(stateArr[i])
        }
      }
      // 循环表头按钮，判断每个按钮的permitArr是否完全包含选中状态，若完全包含，则按钮亮，否则按钮灰
      var toolbar = this.table.toolbar
      for (var j = 0; j < toolbar.length; j++) {
        if (
          uniqueStateArr.every((val) =>
            toolbar[j].permitArr.length <= 0
              ? true
              : toolbar[j].permitArr.includes(val)
          )
        ) {
          toolbar[j].disabled = false
        } else {
          toolbar[j].disabled = true
        }
      }
      // 将选中记录赋值给multipleSelection
      this.multipleSelection = val
    },
    getStateOptionList() {
      return this.stateOptionList
    }
  }
}
</script>
